/*****************************************************
*             EMGO Tranceiver Interface              *
*       (C) 2009 by Marius Petrescu, YO2LOJ          *
*   Released under the GNU General Public License    *
*****************************************************/

/*******************************************************
 History
 ----+------------------+-------------------------------
 Ver.| Date             | Comment
 ----+------------------+-------------------------------
 1.0 |    8.June 2009   | First release
 ----+------------------+-------------------------------
 1.1 |   25.June 2009   | Update freq. table due to
     |                  | NOAA 18/19 frequency change
 ----+------------------+-------------------------------
 1.2 |  9.August 2010   | Manual mode did not work
     |                  | Changed names on unused
     |                  | channels to CH-x
 ----+------------------+-------------------------------
 1.3 | 11.August 2010   | Removed mode without UART 
     |                  | control
     |                  | Cosmetic changes to scan mode
 ----+------------------+-------------------------------
 1.4 | 18.December 2010 | Added active satellite labels
 ----+------------------+-------------------------------
 1.5 | 31.March 2012    | Lbels corrected and updated
 ----+------------------+-------------------------------
*******************************************************/

#include <avr/io.h>
#include <avr/pgmspace.h>

#include "lcd.h"
#include "uart.h"
#include "pll.h"

#define PROG_VERSION	"EMGO 1.5"

#define BUTTON_A	(PINB & 0x04)	//Middle
#define BUTTON_B	(PIND & 0x08)	//Down
#define BUTTON_C	(PIND & 0x04)	//Up
#define SQ_PORT		(PINB & 0x08)	//Squelch
#define SQ_DELAY	200

#define MODE_NORMAL	0
#define MODE_SCAN	1
#define MODE_MANUAL	2

#define FREQ_MAX	14100
#define FREQ_MIN	13400

#define FILTER_CONSTANT	3

#define delay_us(us)	lcd_delay(us)
#define delay_ms(ms)	lcd_delay(1000L * ms)

char button_a = 0;
char button_b = 0;
char button_c = 0;

char old_button_a = 0;
char old_button_b = 0;
char old_button_c = 0;

char idx = 1;
unsigned freq;
unsigned old_freq = 0;

char o_sqopen = 0;

char mode = MODE_NORMAL;
char sdelay = 0;

struct noaa
{
	const prog_char *name;
	const unsigned freq;
};

#define CHAN_NUM	12

/* Strings go to progmem to save RAM */
const prog_char sn0[] PROGMEM = {"CH-1   "};
const prog_char sn1[] PROGMEM = {"NOAA-19"}; /* Working */
const prog_char sn2[] PROGMEM = {"OKE/SIC"}; /* OKEAN-O/SICH-1M sporadic */
const prog_char sn3[] PROGMEM = {"NOAA-17"}; /* stalled */
const prog_char sn4[] PROGMEM = {"NOAA-15"}; /* Working */
const prog_char sn5[] PROGMEM = {"NOAA-18"}; /* Working */
const prog_char sn6[] PROGMEM = {"CH-7   "};
const prog_char sn7[] PROGMEM = {"NOAA-RT"}; /* NOAA beacon */
const prog_char sn8[] PROGMEM = {"CH-9   "};
const prog_char sn9[] PROGMEM = {"CH-10  "};
const prog_char snA[] PROGMEM = {"DCONV-1"};
const prog_char snB[] PROGMEM = {"DCONV-2"};

const struct noaa sats[CHAN_NUM] =
{
	{sn0, 13720},
	{sn1, 13710},
	{sn2, 13740},
	{sn3, 13750},
	{sn4, 13762},
	{sn5, 13791},
	{sn6, 13730},
	{sn7, 13770},
	{sn8, 13780},
	{sn9, 13785},
	{snA, 13400},
	{snB, 14100}
};


void read_buttons(void)
{
	PORTB = 0x0F;	// reenable pullups

	if (!BUTTON_A) button_a = FILTER_CONSTANT;
	else
	{
		if (button_a) button_a--;
	}

	if (!BUTTON_B) button_b = FILTER_CONSTANT;
	else
	{
		if (button_b) button_b--;
	}

	if (!BUTTON_C) button_c = FILTER_CONSTANT;
	else
	{
		if (button_c) button_c--;
	}
}

void set_data(void)
{
	lcd_goto(0,0);
	if (mode == MODE_MANUAL) lcd_puts_P(PSTR("MANUAL "));
	else if (mode == MODE_SCAN) lcd_puts_P(PSTR("SCAN..."));
	else lcd_puts_P(sats[(int)idx].name);
	lcd_sendchar(' ');
	lcd_goto(1,0);
	lcd_sendchar('1');
	lcd_sendchar('0' + freq%10000/1000);
	lcd_sendchar('0' + freq%1000/100);
	lcd_sendchar('.');
	lcd_sendchar('0' + freq%100/10);
	lcd_sendchar('0' + freq%10);
	lcd_sendchar('0');
	pll_send(freq - 1070);
}

int main(void)
{
	char sqopen;

	// set port directions
	DDRB = 0x00;
	DDRD = 0xF0;

	/* activate pullups */
	PORTB = 0x0F;
	PORTD = 0x0F;

	/* init lcd */ 
	lcd_init();
	/* init UART */
	uart_init();
	/* init PLL */
	pll_send(0xC200);

	/* put logo */
  	lcd_puts_P(PSTR(PROG_VERSION));
	lcd_goto(1,0);
	lcd_puts_P(PSTR(" YO2LOJ "));
	delay_ms(2000);

	for(;;)
	{
		read_buttons();

		sqopen = SQ_PORT;
 
		// up
		if (button_c && !old_button_c)
		{
			if (mode == MODE_MANUAL)
			{
				if (freq < FREQ_MAX) freq++;
				else freq = FREQ_MIN;
			}
			else
			{
				idx++;
				if (idx == CHAN_NUM) idx = 0;
			}
		}

		// down
		if (button_b && !old_button_b)
		{
			if (mode == MODE_MANUAL)
			{
				if (freq > FREQ_MIN) freq--;
				else freq = FREQ_MAX;
			}
			else
			{
				if (idx == 0) idx = CHAN_NUM;
				idx--;
			}
		}

		// center (mode switch)
		if (button_a && !old_button_a)
		{
			switch (mode)
			{
			case MODE_NORMAL:
				mode = MODE_SCAN;
				break;
			case MODE_SCAN:
				mode = MODE_MANUAL;
				break;
			case MODE_MANUAL:
				mode = MODE_NORMAL;
				break;
			}
			sdelay = 200;	// reinit scan/blink timer
			old_freq = 0;	// force display update
		}

		//mode blink
		lcd_goto(1, 7);
		if (((mode == MODE_SCAN) || (mode == MODE_NORMAL))&& (sdelay>(SQ_DELAY / 2)))
		{
			if (mode == MODE_NORMAL)	// UART control
			{
				lcd_sendchar(0x7F); // blink "left arrow"	 			
			}
			else if (!sqopen)	// Squelch open
			{
				lcd_sendchar('!'); // blink "!" character
			}
			else
			{
				lcd_sendchar('*'); // blink "*" character
			}
		}
		else
		{
			lcd_sendchar(' ');	// clear in manual mode
		}

		if (mode == MODE_SCAN)	//scan functon
		{
			if (uart_cmd == UART_CMD_DONE)
			{
				mode = MODE_NORMAL;
			}
						
			if ((o_sqopen != sqopen) && (!sqopen))	// show channel name if squelch is open
			{
				lcd_goto(0,0);
				lcd_puts_P(sats[(int)idx].name);
			}
					
			if ((!sdelay) && (sqopen))	// next channel only if squelch is closed
			{
				idx++;
			}

			if (idx >= CHAN_NUM-2) idx = 0;	//don't scan transverter
		}

		if (mode == MODE_NORMAL)	//UART control
		{
			if (uart_cmd == UART_CMD_DONE)
			{
				idx = uart_chan;
				uart_cmd = UART_CMD_NONE;
			}

			if (uart_cmd == UART_CMD_SCAN)
			{
				mode = MODE_SCAN;
				uart_cmd = UART_CMD_NONE;
			}
		}

		// scan and blink delay counter
		if (sdelay) sdelay--;
		else sdelay = SQ_DELAY;

		// update frequency
		if (mode != MODE_MANUAL) freq = sats[(int)idx].freq; 

		// display data
		if (old_freq != freq)	// update only if frequency changed
		{
			set_data();
		}

		// store old values for change detection
		old_freq = freq;
		o_sqopen = sqopen;
		old_button_a = button_a;
		old_button_b = button_b;
		old_button_c = button_c;

	}
	return 0;
}
